@@ -4,14 +4,111 @@ from gvoice import *
 from gvSMSPeople import Ui_MainWindow
 from gvSMSMsg import Ui_MsgWindow
 from gvAccount import *
-import sys
-import os
+import sys, os, time
 import ConfigParser
-import copy
-#from PySide.QtCore import *
-#from PySide.QtGui import *
 from PyQt4.QtCore import *
 from PyQt4.QtGui import *
+import ctypes, sqlite3, uuid
+
+# Information taken from:
+# http://docs.python.org/library/uuid.html#uuid.uuid4
+# http://docs.python.org/library/ctypes.html
+# http://www.dalkescientific.com/writings/NBN/ctypes.html
+# Class constructed from: http://maemo.gitorious.org/maemo-rtcom/rtcom-eventlogger/blobs/master/rtcom-eventlogger/event.h
+# credits to gri for doing the hard work with this
+
+class RTComElEvent(ctypes.Structure):
+    _fields_ = [("_mask", ctypes.c_int),
+                ("fld_id", ctypes.c_int),
+                ("fld_service_id", ctypes.c_int),
+                ("fld_event_type_id", ctypes.c_int),
+                ("fld_storage_time", ctypes.c_long), # time_t
+                ("fld_start_time", ctypes.c_long), # time_t
+                ("fld_end_time", ctypes.c_long), # time_t
+                ("fld_is_read", ctypes.c_int), # Boolean is int
+                ("fld_flags", ctypes.c_int),
+                ("fld_bytes_sent", ctypes.c_int),
+                ("fld_bytes_received", ctypes.c_int),
+                ("fld_local_uid", ctypes.c_char_p),
+                ("fld_local_name", ctypes.c_char_p),
+                ("fld_remote_uid", ctypes.c_char_p),
+                ("fld_remote_name", ctypes.c_char_p),
+                ("fld_remote_ebook_uid", ctypes.c_char_p),
+                ("fld_channel", ctypes.c_char_p),
+                ("fld_free_text", ctypes.c_char_p),
+                ("fld_group_uid", ctypes.c_char_p),
+                ("fld_service", ctypes.c_char_p),
+                ("fld_event_type", ctypes.c_char_p),
+                ("fld_additional_text", ctypes.c_char_p),
+                ("fld_icon_name", ctypes.c_char_p),
+                ("fld_pango_markup", ctypes.c_char_p)]
+
+def put_to_sent_sms(name, number, uniqueid, groupid, message):
+#    rtcom = ctypes.CDLL('librtcom-eventlogger.so.1') # PR 1.2
+    rtcom = ctypes.CDLL('librtcom-eventlogger.so.0') # PR 1.1
+
+    el = ctypes.cast(rtcom.rtcom_el_new(), ctypes.c_void_p)
+    event = ctypes.cast(rtcom.rtcom_el_event_new(), ctypes.POINTER(RTComElEvent))[0]
+
+    event._mask       |= 1 << 18
+    event.fld_service  = 'RTCOM_EL_SERVICE_SMS'
+
+    event._mask       |= 1 << 19
+    event.fld_event_type = 'RTCOM_EL_EVENTTYPE_SMS_OUTBOUND'
+
+    event._mask       |= 1 << 4
+    event.fld_start_time = int(time.time())
+
+    event._mask       |= 1 << 5
+    event.fld_end_time = int(time.time())
+
+    event._mask       |= 1 << 6
+    event.fld_is_read = 1 # TRUE
+
+    event._mask       |= 1 << 10
+    event.fld_remote_ebook_uid = uniqueid
+
+    event._mask       |= 1 << 11
+    event.fld_local_uid = 'ring/tel/ring'
+
+    event._mask       |= 1 << 12
+    event.fld_local_name = '<SelfHandle>'
+
+    event._mask       |= 1 << 13
+    event.fld_remote_uid = number
+
+    event._mask       |= 1 << 14
+    event.fld_remote_name = name
+
+    event._mask       |= 1 << 17
+    event.fld_group_uid = groupid
+
+    event._mask       |= 1 << 16
+    event.fld_free_text = message.data()
+
+    rtcom.rtcom_el_add_event.argtypes = [el, ctypes.c_void_p, ctypes.c_void_p]
+    eventId = rtcom.rtcom_el_add_event(el, ctypes.byref(event), None)
+
+    # the follwing block doesnt seem to be needed for SMS
+    # Add the message that it shows up in conversations
+    rtcom.rtcom_el_add_header.argtypes = [el, ctypes.c_int, ctypes.c_char_p, ctypes.c_char_p, ctypes.c_void_p]
+    rtcom.rtcom_el_add_header(el, eventId, 'message-token', str(uuid.uuid4()), None)
+
+    rtcom.rtcom_el_event_free(ctypes.byref(event))
+
+def write_to_rtcomm(message, number):
+    connection = sqlite3.connect(os.path.join(os.environ['HOME'],
+                                     '.rtcom-eventlogger', 'el.db'))
+    cursor = connection.cursor()
+    cursor.execute('SELECT id FROM Events')
+    id_array = cursor.fetchall()
+    seq_id = max(id_array)[0] + 1
+        
+    strippednumber = number[-7:]
+
+    own_number = ""
+        
+    put_to_sent_sms(own_number, number, 0, strippednumber, message)
 
 def octify(str):
     '''     
@@ -20,9 +117,10 @@ def octify(str):
     '''                                       
 
     bytes = map(ord, str)
-    bitsconsumed = 0     
-    referencebit = 7     
-    octets = []          
+    bitsconsumed = 0
+    referencebit = 7
+    octets = []
+    bitstocopy = 0
 
     while len(bytes):
             byte = bytes.pop(0)
@@ -103,6 +201,7 @@ def cleansenumber(number):
         cleannumber = cleannumber.replace(c, "")
     return(cleannumber)
 
+
 class SMS(object):
     '''
     Sends sms messages
@@ -110,7 +209,8 @@ class SMS(object):
     
     def __init__(self, msg, number):
         self.pdustring = createPDUstring(number, msg)
-        
+        write_to_rtcomm(msg, number)
+
     def send(self):
         self.__dbus_send(self.pdustring)
 
@@ -143,23 +243,26 @@ class MainWindow(QMainWindow, Ui_MainWindow):
         #pyside 0.2.2 bug? auto-connect of signals is not working
         #self.connect(self.pushButtonSend, SIGNAL("released()"), self.on_pushButtonSend_released)
 
-    def load_config(self):
-        try:
-            config = ConfigParser.ConfigParser()
-            config.readfp(open(os.path.expanduser('~') + '/.gvSMS'))
+    def load_config(self, loadconfig=True):
+        if loadconfig == True:
             try:
-                email = config.get("credentials", "email")
-                #self.email = base64.b64decode(email)
-                self.email = email
-                password = config.get("credentials", "password")
-                #self.password = base64.b64decode(password)
-                self.password = password
-            except ConfigParser.NoSectionError:
-                   self.prompt()
-            except ConfigParser.NoOptionError:
-                   self.prompt()
-        except IOError:
-            # couldn't find the file set uid so we can prompt info
+                config = ConfigParser.ConfigParser()
+                config.readfp(open(os.path.expanduser('~') + '/.gvSMS'))
+                try:
+                    email = config.get("credentials", "email")
+                    #self.email = base64.b64decode(email)
+                    self.email = email
+                    password = config.get("credentials", "password")
+                    #self.password = base64.b64decode(password)
+                    self.password = password
+                except ConfigParser.NoSectionError:
+                    self.prompt()
+                except ConfigParser.NoOptionError:
+                    self.prompt()
+            except IOError:
+                # couldn't find the file set uid so we can prompt info
+                self.prompt()
+        else:
             self.prompt()
         if len(self.email) > 0 and len(self.password) > 0:
             self.plainTextEditStatus.appendPlainText("Logging in...")
@@ -177,25 +280,26 @@ class MainWindow(QMainWindow, Ui_MainWindow):
     		self.plainTextEditStatus.appendPlainText('failed to write config file')
 
     def prompt(self):
-        if len(self.email) == 0 or len(self.password) == 0:
-            mydialog = QtGui.QDialog(self)
-            uidialog = Ui_Dialog()
-            uidialog.setupUi(mydialog)
-            uidialog.lineEditPassword.setEchoMode(QLineEdit.Password)
-            mydialog.show()
-            if mydialog.exec_():
-                dialogaccount = uidialog.lineEditAccount.text()
-                dialogpassword = uidialog.lineEditPassword.text()
-                if len(dialogaccount) > 0 and len(dialogpassword) > 0:
-                    self.email = dialogaccount
-                    self.password = dialogpassword
-                    if uidialog.checkBox.isChecked():
-                        self.save_config()
-                    else:
-                        try:
-                            f = open(os.path.expanduser('~') + '/.gvSMS', 'w')
-                        except IOError, e:
-                            self.plainTextEditStatus.appendPlainText('failed to remove config file')
+        mydialog = QtGui.QDialog(self)
+        uidialog = Ui_Dialog()
+        uidialog.setupUi(mydialog)
+        uidialog.lineEditPassword.setEchoMode(QLineEdit.Password)
+        uidialog.lineEditAccount.insert(self.email)
+        uidialog.lineEditPassword.insert(self.password)
+        mydialog.show()
+        if mydialog.exec_():
+            dialogaccount = uidialog.lineEditAccount.text()
+            dialogpassword = uidialog.lineEditPassword.text()
+            if len(dialogaccount) > 0 and len(dialogpassword) > 0:
+                self.email = dialogaccount
+                self.password = dialogpassword
+                if uidialog.checkBox.isChecked():
+                    self.save_config()
+                else:
+                    try:
+                        f = open(os.path.expanduser('~') + '/.gvSMS', 'w')
+                    except IOError, e:
+                        self.plainTextEditStatus.appendPlainText('failed to remove config file')
 
         if len(self.email) == 0 or len(self.password) == 0:
             self.plainTextEditStatus.appendPlainText("No credentials supplied")
@@ -205,17 +309,18 @@ class MainWindow(QMainWindow, Ui_MainWindow):
         if not self.gv.logged_in:
            self.plainTextEditStatus.appendPlainText("Could not log in with provided credentials")
         else:
-             self.plainTextEditStatus.appendPlainText("Login successful!")
-             # Use the ContactLoader to download Google Contacts
-             contact_loader = ContactLoader(self.gv.opener)
-             # Use the ContactSelector to select the group and
-             # final list of contacts to contact
-             self.contact_selector = ContactSelector(contact_loader.contacts_by_group_list)
-             # save a copy to reload after any deletes
-             #self.save_contacts_by_group_list = copy.deepcopy(contact_loader.contacts_by_group_list)
-             group_list = self.contact_selector.get_group_list()
-             for group_item in group_list:
-                 self.listWidgetGroups.addItem(group_item[1])
+            self.listWidgetGroups.clear()
+            self.plainTextEditStatus.appendPlainText("Login successful!")
+            # Use the ContactLoader to download Google Contacts
+            contact_loader = ContactLoader(self.gv.opener)
+            # Use the ContactSelector to select the group and
+            # final list of contacts to contact
+            self.contact_selector = ContactSelector(contact_loader.contacts_by_group_list)
+            # save a copy to reload after any deletes
+            #self.save_contacts_by_group_list = copy.deepcopy(contact_loader.contacts_by_group_list)
+            group_list = self.contact_selector.get_group_list()
+            for group_item in group_list:
+                self.listWidgetGroups.addItem(group_item[1])
 
     def on_listWidgetGroups_itemClicked(self, item):
         # reload original list in case user has deleted any
@@ -260,9 +365,10 @@ class MainWindow(QMainWindow, Ui_MainWindow):
 
     @QtCore.pyqtSlot()
     def on_actionLog_In_triggered(self):
-        self.email = ""
-        self.password = ""
-        self.load_config()
+        if not self.gv.logged_in:
+            self.load_config(False)
+        else:
+            self.plainTextEditStatus.appendPlainText("Currently logged in") 
         
     def on_pushButtonNext_released(self):
         self.msgWindow.show()
@@ -309,6 +415,7 @@ class MsgWindow(QMainWindow, Ui_MsgWindow):
                         self.plainTextEditStatus.appendPlainText(message + "Success!")
                     else:
                         self.plainTextEditStatus.appendPlainText(message + "Failed!!")
+                    time.sleep(3)
             i = i + 1
             
         '''
@@ -340,9 +447,6 @@ class MsgWindow(QMainWindow, Ui_MsgWindow):
         else:
              self.pushButtonSend.setEnabled(False)
 
-    @QtCore.pyqtSlot()
-    def on_actionLog_In_triggered(self):
-        self.load_config()
 
 if __name__ == "__main__":
     import sys
